@@ -1,13 +1,11 @@
 /*
   +----------------------------------------------------------------------+
-  | PHP Version 7                                                        |
-  +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2017 The PHP Group                                |
+  | Copyright (c) The PHP Group                                          |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
-  | http://www.php.net/license/3_01.txt                                  |
+  | https://www.php.net/license/3_01.txt                                 |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
@@ -17,8 +15,6 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
-
 #include "php_hash.h"
 #include "php_hash_sha.h"
 
@@ -67,353 +63,31 @@ static void SHADecode32(uint32_t *output, const unsigned char *input, unsigned i
 /* }}} */
 
 const php_hash_ops php_hash_sha1_ops = {
-	(php_hash_init_func_t) PHP_SHA1Init,
+	"sha1",
+	(php_hash_init_func_t) PHP_SHA1InitArgs,
 	(php_hash_update_func_t) PHP_SHA1Update,
 	(php_hash_final_func_t) PHP_SHA1Final,
-	(php_hash_copy_func_t) php_hash_copy,
+	php_hash_copy,
+	php_hash_serialize,
+	php_hash_unserialize,
+	PHP_SHA1_SPEC,
 	20,
 	64,
 	sizeof(PHP_SHA1_CTX),
 	1
 };
 
-#ifdef PHP_HASH_SHA1_NOT_IN_CORE
-
-PHP_HASH_API void make_sha1_digest(char *sha1str, unsigned char *digest)
-{
-	php_hash_bin2hex(sha1str, digest, 20);
-	sha1str[40] = '\0';
-}
-
-/* {{{ proto string sha1(string str [, bool raw_output])
-   Calculate the sha1 hash of a string */
-PHP_FUNCTION(sha1)
-{
-	char *arg;
-	size_t arg_len;
-	zend_bool raw_output = 0;
-	char sha1str[41];
-	PHP_SHA1_CTX context;
-	unsigned char digest[20];
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s|b", &arg, &arg_len, &raw_output) == FAILURE) {
-		return;
-	}
-
-	sha1str[0] = '\0';
-	PHP_SHA1Init(&context);
-	PHP_SHA1Update(&context, arg, arg_len);
-	PHP_SHA1Final(digest, &context);
-	if (raw_output) {
-		RETURN_STRINGL(digest, 20);
-	} else {
-		make_sha1_digest(sha1str, digest);
-		RETVAL_STRING(sha1str);
-	}
-
-}
-
-/* }}} */
-
-/* {{{ proto string sha1_file(string filename [, bool raw_output])
-   Calculate the sha1 hash of given filename */
-PHP_FUNCTION(sha1_file)
-{
-	char          *arg;
-	size_t        arg_len;
-	zend_bool raw_output = 0;
-	char          sha1str[41];
-	unsigned char buf[1024];
-	unsigned char digest[20];
-	PHP_SHA1_CTX   context;
-	int           n;
-	php_stream    *stream;
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "p|b", &arg, &arg_len, &raw_output) == FAILURE) {
-		return;
-	}
-
-	stream = php_stream_open_wrapper(arg, "rb", REPORT_ERRORS, NULL);
-	if (!stream) {
-		RETURN_FALSE;
-	}
-
-	PHP_SHA1Init(&context);
-
-	while ((n = php_stream_read(stream, buf, sizeof(buf))) > 0) {
-		PHP_SHA1Update(&context, buf, n);
-	}
-
-	PHP_SHA1Final(digest, &context);
-
-	php_stream_close(stream);
-
-	if (n<0) {
-		RETURN_FALSE;
-	}
-
-	if (raw_output) {
-		RETURN_STRINGL(digest, 20);
-	} else {
-		make_sha1_digest(sha1str, digest);
-		RETVAL_STRING(sha1str);
-	}
-}
-/* }}} */
-
-/* F, G, H and I are basic SHA1 functions.
- */
-#define F(x, y, z) ((z) ^ ((x) & ((y) ^ (z))))
-#define G(x, y, z) ((x) ^ (y) ^ (z))
-#define H(x, y, z) (((x) & (y)) | ((z) & ((x) | (y))))
-#define I(x, y, z) ((x) ^ (y) ^ (z))
-
-/* ROTATE_LEFT rotates x left n bits.
- */
-#define ROTATE_LEFT(x, n) (((x) << (n)) | ((x) >> (32-(n))))
-
-/* W[i]
- */
-#define W(i) ( tmp=x[(i-3)&15]^x[(i-8)&15]^x[(i-14)&15]^x[i&15], \
-	(x[i&15]=ROTATE_LEFT(tmp, 1)) )
-
-/* FF, GG, HH, and II transformations for rounds 1, 2, 3, and 4.
- */
-#define FF(a, b, c, d, e, w) { \
- (e) += F ((b), (c), (d)) + (w) + (uint32_t)(0x5A827999); \
- (e) += ROTATE_LEFT ((a), 5); \
- (b) = ROTATE_LEFT((b), 30); \
-  }
-#define GG(a, b, c, d, e, w) { \
- (e) += G ((b), (c), (d)) + (w) + (uint32_t)(0x6ED9EBA1); \
- (e) += ROTATE_LEFT ((a), 5); \
- (b) = ROTATE_LEFT((b), 30); \
-  }
-#define HH(a, b, c, d, e, w) { \
- (e) += H ((b), (c), (d)) + (w) + (uint32_t)(0x8F1BBCDC); \
- (e) += ROTATE_LEFT ((a), 5); \
- (b) = ROTATE_LEFT((b), 30); \
-  }
-#define II(a, b, c, d, e, w) { \
- (e) += I ((b), (c), (d)) + (w) + (uint32_t)(0xCA62C1D6); \
- (e) += ROTATE_LEFT ((a), 5); \
- (b) = ROTATE_LEFT((b), 30); \
-  }
-
-
-/* {{{ PHP_SHA1Init
- * SHA1 initialization. Begins an SHA1 operation, writing a new context.
- */
-PHP_HASH_API void PHP_SHA1Init(PHP_SHA1_CTX * context)
-{
-	context->count[0] = context->count[1] = 0;
-	/* Load magic initialization constants.
-	 */
-	context->state[0] = 0x67452301;
-	context->state[1] = 0xefcdab89;
-	context->state[2] = 0x98badcfe;
-	context->state[3] = 0x10325476;
-	context->state[4] = 0xc3d2e1f0;
-}
-/* }}} */
-
-/* {{{ SHA1Transform
- * SHA1 basic transformation. Transforms state based on block.
- */
-static void SHA1Transform(uint32_t state[5], const unsigned char block[64])
-{
-	uint32_t a = state[0], b = state[1], c = state[2];
-	uint32_t d = state[3], e = state[4], x[16], tmp;
-
-	SHADecode32(x, block, 64);
-
-	/* Round 1 */
-	FF(a, b, c, d, e, x[0]);   /* 1 */
-	FF(e, a, b, c, d, x[1]);   /* 2 */
-	FF(d, e, a, b, c, x[2]);   /* 3 */
-	FF(c, d, e, a, b, x[3]);   /* 4 */
-	FF(b, c, d, e, a, x[4]);   /* 5 */
-	FF(a, b, c, d, e, x[5]);   /* 6 */
-	FF(e, a, b, c, d, x[6]);   /* 7 */
-	FF(d, e, a, b, c, x[7]);   /* 8 */
-	FF(c, d, e, a, b, x[8]);   /* 9 */
-	FF(b, c, d, e, a, x[9]);   /* 10 */
-	FF(a, b, c, d, e, x[10]);  /* 11 */
-	FF(e, a, b, c, d, x[11]);  /* 12 */
-	FF(d, e, a, b, c, x[12]);  /* 13 */
-	FF(c, d, e, a, b, x[13]);  /* 14 */
-	FF(b, c, d, e, a, x[14]);  /* 15 */
-	FF(a, b, c, d, e, x[15]);  /* 16 */
-	FF(e, a, b, c, d, W(16));  /* 17 */
-	FF(d, e, a, b, c, W(17));  /* 18 */
-	FF(c, d, e, a, b, W(18));  /* 19 */
-	FF(b, c, d, e, a, W(19));  /* 20 */
-
-	/* Round 2 */
-	GG(a, b, c, d, e, W(20));  /* 21 */
-	GG(e, a, b, c, d, W(21));  /* 22 */
-	GG(d, e, a, b, c, W(22));  /* 23 */
-	GG(c, d, e, a, b, W(23));  /* 24 */
-	GG(b, c, d, e, a, W(24));  /* 25 */
-	GG(a, b, c, d, e, W(25));  /* 26 */
-	GG(e, a, b, c, d, W(26));  /* 27 */
-	GG(d, e, a, b, c, W(27));  /* 28 */
-	GG(c, d, e, a, b, W(28));  /* 29 */
-	GG(b, c, d, e, a, W(29));  /* 30 */
-	GG(a, b, c, d, e, W(30));  /* 31 */
-	GG(e, a, b, c, d, W(31));  /* 32 */
-	GG(d, e, a, b, c, W(32));  /* 33 */
-	GG(c, d, e, a, b, W(33));  /* 34 */
-	GG(b, c, d, e, a, W(34));  /* 35 */
-	GG(a, b, c, d, e, W(35));  /* 36 */
-	GG(e, a, b, c, d, W(36));  /* 37 */
-	GG(d, e, a, b, c, W(37));  /* 38 */
-	GG(c, d, e, a, b, W(38));  /* 39 */
-	GG(b, c, d, e, a, W(39));  /* 40 */
-
-	/* Round 3 */
-	HH(a, b, c, d, e, W(40));  /* 41 */
-	HH(e, a, b, c, d, W(41));  /* 42 */
-	HH(d, e, a, b, c, W(42));  /* 43 */
-	HH(c, d, e, a, b, W(43));  /* 44 */
-	HH(b, c, d, e, a, W(44));  /* 45 */
-	HH(a, b, c, d, e, W(45));  /* 46 */
-	HH(e, a, b, c, d, W(46));  /* 47 */
-	HH(d, e, a, b, c, W(47));  /* 48 */
-	HH(c, d, e, a, b, W(48));  /* 49 */
-	HH(b, c, d, e, a, W(49));  /* 50 */
-	HH(a, b, c, d, e, W(50));  /* 51 */
-	HH(e, a, b, c, d, W(51));  /* 52 */
-	HH(d, e, a, b, c, W(52));  /* 53 */
-	HH(c, d, e, a, b, W(53));  /* 54 */
-	HH(b, c, d, e, a, W(54));  /* 55 */
-	HH(a, b, c, d, e, W(55));  /* 56 */
-	HH(e, a, b, c, d, W(56));  /* 57 */
-	HH(d, e, a, b, c, W(57));  /* 58 */
-	HH(c, d, e, a, b, W(58));  /* 59 */
-	HH(b, c, d, e, a, W(59));  /* 60 */
-
-	/* Round 4 */
-	II(a, b, c, d, e, W(60));  /* 61 */
-	II(e, a, b, c, d, W(61));  /* 62 */
-	II(d, e, a, b, c, W(62));  /* 63 */
-	II(c, d, e, a, b, W(63));  /* 64 */
-	II(b, c, d, e, a, W(64));  /* 65 */
-	II(a, b, c, d, e, W(65));  /* 66 */
-	II(e, a, b, c, d, W(66));  /* 67 */
-	II(d, e, a, b, c, W(67));  /* 68 */
-	II(c, d, e, a, b, W(68));  /* 69 */
-	II(b, c, d, e, a, W(69));  /* 70 */
-	II(a, b, c, d, e, W(70));  /* 71 */
-	II(e, a, b, c, d, W(71));  /* 72 */
-	II(d, e, a, b, c, W(72));  /* 73 */
-	II(c, d, e, a, b, W(73));  /* 74 */
-	II(b, c, d, e, a, W(74));  /* 75 */
-	II(a, b, c, d, e, W(75));  /* 76 */
-	II(e, a, b, c, d, W(76));  /* 77 */
-	II(d, e, a, b, c, W(77));  /* 78 */
-	II(c, d, e, a, b, W(78));  /* 79 */
-	II(b, c, d, e, a, W(79));  /* 80 */
-
-	state[0] += a;
-	state[1] += b;
-	state[2] += c;
-	state[3] += d;
-	state[4] += e;
-
-	/* Zeroize sensitive information. */
-	ZEND_SECURE_ZERO((unsigned char*) x, sizeof(x));
-}
-/* }}} */
-
-/* {{{ PHP_SHA1Update
-   SHA1 block update operation. Continues an SHA1 message-digest
-   operation, processing another message block, and updating the
-   context.
- */
-PHP_HASH_API void PHP_SHA1Update(PHP_SHA1_CTX * context, const unsigned char *input,
-			   unsigned int inputLen)
-{
-	unsigned int i, index, partLen;
-
-	/* Compute number of bytes mod 64 */
-	index = (unsigned int) ((context->count[0] >> 3) & 0x3F);
-
-	/* Update number of bits */
-	if ((context->count[0] += ((uint32_t) inputLen << 3))
-		< ((uint32_t) inputLen << 3))
-		context->count[1]++;
-	context->count[1] += ((uint32_t) inputLen >> 29);
-
-	partLen = 64 - index;
-
-	/* Transform as many times as possible.
-	 */
-	if (inputLen >= partLen) {
-		memcpy
-			((unsigned char*) & context->buffer[index], (unsigned char*) input, partLen);
-		SHA1Transform(context->state, context->buffer);
-
-		for (i = partLen; i + 63 < inputLen; i += 64)
-			SHA1Transform(context->state, &input[i]);
-
-		index = 0;
-	} else
-		i = 0;
-
-	/* Buffer remaining input */
-	memcpy
-		((unsigned char*) & context->buffer[index], (unsigned char*) & input[i],
-		 inputLen - i);
-}
-/* }}} */
-
-/* {{{ PHP_SHA1Final
-   SHA1 finalization. Ends an SHA1 message-digest operation, writing the
-   the message digest and zeroizing the context.
- */
-PHP_HASH_API void PHP_SHA1Final(unsigned char digest[20], PHP_SHA1_CTX * context)
-{
-	unsigned char bits[8];
-	unsigned int index, padLen;
-
-	/* Save number of bits */
-	bits[7] = context->count[0] & 0xFF;
-	bits[6] = (context->count[0] >> 8) & 0xFF;
-	bits[5] = (context->count[0] >> 16) & 0xFF;
-	bits[4] = (context->count[0] >> 24) & 0xFF;
-	bits[3] = context->count[1] & 0xFF;
-	bits[2] = (context->count[1] >> 8) & 0xFF;
-	bits[1] = (context->count[1] >> 16) & 0xFF;
-	bits[0] = (context->count[1] >> 24) & 0xFF;
-
-	/* Pad out to 56 mod 64.
-	 */
-	index = (unsigned int) ((context->count[0] >> 3) & 0x3f);
-	padLen = (index < 56) ? (56 - index) : (120 - index);
-	PHP_SHA1Update(context, PADDING, padLen);
-
-	/* Append length (before padding) */
-	PHP_SHA1Update(context, bits, 8);
-
-	/* Store state in digest */
-	SHAEncode32(digest, context->state, 20);
-
-	/* Zeroize sensitive information.
-	 */
-	ZEND_SECURE_ZERO((unsigned char*) context, sizeof(*context));
-}
-/* }}} */
-
-#endif /* PHP_HASH_SHA1_NOT_IN_CORE */
-
 /* sha224/sha256 */
 
 const php_hash_ops php_hash_sha256_ops = {
-	(php_hash_init_func_t) PHP_SHA256Init,
+	"sha256",
+	(php_hash_init_func_t) PHP_SHA256InitArgs,
 	(php_hash_update_func_t) PHP_SHA256Update,
 	(php_hash_final_func_t) PHP_SHA256Final,
-	(php_hash_copy_func_t) php_hash_copy,
+	php_hash_copy,
+	php_hash_serialize,
+	php_hash_unserialize,
+	PHP_SHA256_SPEC,
 	32,
 	64,
 	sizeof(PHP_SHA256_CTX),
@@ -421,10 +95,14 @@ const php_hash_ops php_hash_sha256_ops = {
 };
 
 const php_hash_ops php_hash_sha224_ops = {
-	(php_hash_init_func_t) PHP_SHA224Init,
+	"sha224",
+	(php_hash_init_func_t) PHP_SHA224InitArgs,
 	(php_hash_update_func_t) PHP_SHA224Update,
 	(php_hash_final_func_t) PHP_SHA224Final,
-	(php_hash_copy_func_t) php_hash_copy,
+	php_hash_copy,
+	php_hash_serialize,
+	php_hash_unserialize,
+	PHP_SHA224_SPEC,
 	28,
 	64,
 	sizeof(PHP_SHA224_CTX),
@@ -458,10 +136,10 @@ static const uint32_t SHA256_K[64] = {
 	0x19a4c116, 0x1e376c08, 0x2748774c, 0x34b0bcb5, 0x391c0cb3, 0x4ed8aa4a, 0x5b9cca4f, 0x682e6ff3,
 	0x748f82ee, 0x78a5636f, 0x84c87814, 0x8cc70208, 0x90befffa, 0xa4506ceb, 0xbef9a3f7, 0xc67178f2 };
 
-/* {{{ PHP_SHA256Init
+/* {{{ PHP_SHA256InitArgs
  * SHA256 initialization. Begins an SHA256 operation, writing a new context.
  */
-PHP_HASH_API void PHP_SHA256Init(PHP_SHA256_CTX * context)
+PHP_HASH_API void PHP_SHA256InitArgs(PHP_SHA256_CTX * context, ZEND_ATTRIBUTE_UNUSED HashTable *args)
 {
 	context->count[0] = context->count[1] = 0;
 	/* Load magic initialization constants.
@@ -518,10 +196,10 @@ static void SHA256Transform(uint32_t state[8], const unsigned char block[64])
 }
 /* }}} */
 
-/* {{{ PHP_SHA224Init
+/* {{{ PHP_SHA224InitArgs
  * SHA224 initialization. Begins an SHA224 operation, writing a new context.
  */
-PHP_HASH_API void PHP_SHA224Init(PHP_SHA224_CTX * context)
+PHP_HASH_API void PHP_SHA224InitArgs(PHP_SHA224_CTX * context, ZEND_ATTRIBUTE_UNUSED HashTable *args)
 {
 	context->count[0] = context->count[1] = 0;
 	/* Load magic initialization constants.
@@ -542,7 +220,7 @@ PHP_HASH_API void PHP_SHA224Init(PHP_SHA224_CTX * context)
    operation, processing another message block, and updating the
    context.
  */
-PHP_HASH_API void PHP_SHA224Update(PHP_SHA224_CTX * context, const unsigned char *input, unsigned int inputLen)
+PHP_HASH_API void PHP_SHA224Update(PHP_SHA224_CTX * context, const unsigned char *input, size_t inputLen)
 {
 	unsigned int i, index, partLen;
 
@@ -619,7 +297,7 @@ PHP_HASH_API void PHP_SHA224Final(unsigned char digest[28], PHP_SHA224_CTX * con
    operation, processing another message block, and updating the
    context.
  */
-PHP_HASH_API void PHP_SHA256Update(PHP_SHA256_CTX * context, const unsigned char *input, unsigned int inputLen)
+PHP_HASH_API void PHP_SHA256Update(PHP_SHA256_CTX * context, const unsigned char *input, size_t inputLen)
 {
 	unsigned int i, index, partLen;
 
@@ -767,10 +445,10 @@ static void SHADecode64(uint64_t *output, const unsigned char *input, unsigned i
 }
 /* }}} */
 
-/* {{{ PHP_SHA384Init
+/* {{{ PHP_SHA384InitArgs
  * SHA384 initialization. Begins an SHA384 operation, writing a new context.
  */
-PHP_HASH_API void PHP_SHA384Init(PHP_SHA384_CTX * context)
+PHP_HASH_API void PHP_SHA384InitArgs(PHP_SHA384_CTX * context, ZEND_ATTRIBUTE_UNUSED HashTable *args)
 {
 	context->count[0] = context->count[1] = 0;
 	/* Load magic initialization constants.
@@ -833,9 +511,9 @@ static void SHA512Transform(uint64_t state[8], const unsigned char block[128])
    operation, processing another message block, and updating the
    context.
  */
-PHP_HASH_API void PHP_SHA384Update(PHP_SHA384_CTX * context, const unsigned char *input, unsigned int inputLen)
+PHP_HASH_API void PHP_SHA384Update(PHP_SHA384_CTX * context, const unsigned char *input, size_t inputLen)
 {
-	unsigned int i, index, partLen;
+	unsigned int i = 0, index, partLen;
 
 	/* Compute number of bytes mod 128 */
 	index = (unsigned int) ((context->count[0] >> 3) & 0x7F);
@@ -859,8 +537,6 @@ PHP_HASH_API void PHP_SHA384Update(PHP_SHA384_CTX * context, const unsigned char
 		}
 
 		index = 0;
-	} else {
-		i = 0;
 	}
 
 	/* Buffer remaining input */
@@ -914,20 +590,24 @@ PHP_HASH_API void PHP_SHA384Final(unsigned char digest[48], PHP_SHA384_CTX * con
 /* }}} */
 
 const php_hash_ops php_hash_sha384_ops = {
-	(php_hash_init_func_t) PHP_SHA384Init,
+	"sha384",
+	(php_hash_init_func_t) PHP_SHA384InitArgs,
 	(php_hash_update_func_t) PHP_SHA384Update,
 	(php_hash_final_func_t) PHP_SHA384Final,
-	(php_hash_copy_func_t) php_hash_copy,
+	php_hash_copy,
+	php_hash_serialize,
+	php_hash_unserialize,
+	PHP_SHA384_SPEC,
 	48,
 	128,
 	sizeof(PHP_SHA384_CTX),
 	1
 };
 
-/* {{{ PHP_SHA512Init
+/* {{{ PHP_SHA512InitArgs
  * SHA512 initialization. Begins an SHA512 operation, writing a new context.
  */
-PHP_HASH_API void PHP_SHA512Init(PHP_SHA512_CTX * context)
+PHP_HASH_API void PHP_SHA512InitArgs(PHP_SHA512_CTX * context, ZEND_ATTRIBUTE_UNUSED HashTable *args)
 {
 	context->count[0] = context->count[1] = 0;
 	/* Load magic initialization constants.
@@ -943,10 +623,10 @@ PHP_HASH_API void PHP_SHA512Init(PHP_SHA512_CTX * context)
 }
 /* }}} */
 
-/* {{{ PHP_SHA512_256Init
+/* {{{ PHP_SHA512_256InitArgs
  * SHA512/245 initialization. Identical algorithm to SHA512, using alternate initval and truncation
  */
-PHP_HASH_API void PHP_SHA512_256Init(PHP_SHA512_CTX * context)
+PHP_HASH_API void PHP_SHA512_256InitArgs(PHP_SHA512_CTX * context, ZEND_ATTRIBUTE_UNUSED HashTable *args)
 {
 	context->count[0] = context->count[1] = 0;
 
@@ -961,10 +641,10 @@ PHP_HASH_API void PHP_SHA512_256Init(PHP_SHA512_CTX * context)
 }
 /* }}} */
 
-/* {{{ PHP_SHA512_224Init
+/* {{{ PHP_SHA512_224InitArgs
  * SHA512/224 initialization. Identical algorithm to SHA512, using alternate initval and truncation
  */
-PHP_HASH_API void PHP_SHA512_224Init(PHP_SHA512_CTX * context)
+PHP_HASH_API void PHP_SHA512_224InitArgs(PHP_SHA512_CTX * context, ZEND_ATTRIBUTE_UNUSED HashTable *args)
 {
         context->count[0] = context->count[1] = 0;
 
@@ -984,7 +664,7 @@ PHP_HASH_API void PHP_SHA512_224Init(PHP_SHA512_CTX * context)
    operation, processing another message block, and updating the
    context.
  */
-PHP_HASH_API void PHP_SHA512Update(PHP_SHA512_CTX * context, const unsigned char *input, unsigned int inputLen)
+PHP_HASH_API void PHP_SHA512Update(PHP_SHA512_CTX * context, const unsigned char *input, size_t inputLen)
 {
 	unsigned int i, index, partLen;
 
@@ -1087,10 +767,14 @@ PHP_HASH_API void PHP_SHA512_224Final(unsigned char digest[28], PHP_SHA512_CTX *
 /* }}} */
 
 const php_hash_ops php_hash_sha512_ops = {
-	(php_hash_init_func_t) PHP_SHA512Init,
+	"sha512",
+	(php_hash_init_func_t) PHP_SHA512InitArgs,
 	(php_hash_update_func_t) PHP_SHA512Update,
 	(php_hash_final_func_t) PHP_SHA512Final,
-	(php_hash_copy_func_t) php_hash_copy,
+	php_hash_copy,
+	php_hash_serialize,
+	php_hash_unserialize,
+	PHP_SHA512_SPEC,
 	64,
 	128,
 	sizeof(PHP_SHA512_CTX),
@@ -1098,10 +782,14 @@ const php_hash_ops php_hash_sha512_ops = {
 };
 
 const php_hash_ops php_hash_sha512_256_ops = {
-	(php_hash_init_func_t) PHP_SHA512_256Init,
+	"sha512/256",
+	(php_hash_init_func_t) PHP_SHA512_256InitArgs,
 	(php_hash_update_func_t) PHP_SHA512_256Update,
 	(php_hash_final_func_t) PHP_SHA512_256Final,
-	(php_hash_copy_func_t) php_hash_copy,
+	php_hash_copy,
+	php_hash_serialize,
+	php_hash_unserialize,
+	PHP_SHA512_SPEC,
 	32,
 	128,
 	sizeof(PHP_SHA512_CTX),
@@ -1109,21 +797,16 @@ const php_hash_ops php_hash_sha512_256_ops = {
 };
 
 const php_hash_ops php_hash_sha512_224_ops = {
-	(php_hash_init_func_t) PHP_SHA512_224Init,
+	"sha512/224",
+	(php_hash_init_func_t) PHP_SHA512_224InitArgs,
 	(php_hash_update_func_t) PHP_SHA512_224Update,
 	(php_hash_final_func_t) PHP_SHA512_224Final,
-	(php_hash_copy_func_t) php_hash_copy,
+	php_hash_copy,
+	php_hash_serialize,
+	php_hash_unserialize,
+	PHP_SHA512_SPEC,
 	28,
 	128,
 	sizeof(PHP_SHA512_CTX),
 	1
 };
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: sw=4 ts=4 fdm=marker
- * vim<600: sw=4 ts=4
- */
