@@ -1,11 +1,9 @@
 /*
-   +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -141,7 +139,7 @@ zend_string* intl_convert_utf16_to_utf8(
 	u_strToUTF8( ZSTR_VAL(dst), dst_len, NULL, src, src_len, status );
 	if( U_FAILURE( *status ) )
 	{
-		zend_string_free(dst);
+		zend_string_efree(dst);
 		return NULL;
 	}
 
@@ -152,12 +150,3 @@ zend_string* intl_convert_utf16_to_utf8(
 	return dst;
 }
 /* }}} */
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
