@@ -1,13 +1,11 @@
 /*
-   +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -19,12 +17,24 @@
 #include "php.h"
 #include "basic_functions.h"
 #include "crc32.h"
+#include "crc32_x86.h"
 
-#if defined(__aarch64__) && defined(HAVE_SYS_AUXV_H)
+#if HAVE_AARCH64_CRC32
 # include <arm_acle.h>
 # if defined(__linux__)
 #  include <sys/auxv.h>
 #  include <asm/hwcap.h>
+# elif defined(__APPLE__)
+#  include <sys/sysctl.h>
+# elif defined(__FreeBSD__)
+#  include <sys/auxv.h>
+
+static unsigned long getauxval(unsigned long key) {
+	unsigned long ret = 0;
+	if (elf_aux_info(key, &ret, sizeof(ret)) != 0)
+		return 0;
+	return ret;
+}
 # endif
 
 static inline int has_crc32_insn() {
@@ -38,15 +48,22 @@ static inline int has_crc32_insn() {
 # elif defined(HWCAP2_CRC32)
 	res = getauxval(AT_HWCAP2) & HWCAP2_CRC32;
 	return res;
+# elif defined(__APPLE__)
+	size_t reslen = sizeof(res);
+	if (sysctlbyname("hw.optional.armv8_crc32", &res, &reslen, NULL, 0) < 0)
+		res = 0;
+	return res;
 # else
 	res = 0;
 	return res;
 # endif
 }
 
-# pragma GCC push_options
-# pragma GCC target ("+nothing+crc")
-static uint32_t crc32_aarch64(uint32_t crc, char *p, size_t nr) {
+# if defined(__GNUC__) && !defined(__clang__)
+#  pragma GCC push_options
+#  pragma GCC target ("+nothing+crc")
+# endif
+static uint32_t crc32_aarch64(uint32_t crc, const char *p, size_t nr) {
 	while (nr >= sizeof(uint64_t)) {
 		crc = __crc32d(crc, *(uint64_t *)p);
 		p += sizeof(uint64_t);
@@ -67,34 +84,68 @@ static uint32_t crc32_aarch64(uint32_t crc, char *p, size_t nr) {
 	}
 	return crc;
 }
-# pragma GCC pop_options
+# if defined(__GNUC__) && !defined(__clang__)
+#  pragma GCC pop_options
+# endif
 #endif
 
-/* {{{ proto string crc32(string str)
-   Calculate the crc32 polynomial of a string */
-PHP_NAMED_FUNCTION(php_if_crc32)
+PHPAPI uint32_t php_crc32_bulk_update(uint32_t crc, const char *p, size_t nr)
+{
+#if HAVE_AARCH64_CRC32
+	if (has_crc32_insn()) {
+		crc = crc32_aarch64(crc, p, nr);
+		return crc;
+	}
+#endif
+
+#if ZEND_INTRIN_SSE4_2_PCLMUL_NATIVE || ZEND_INTRIN_SSE4_2_PCLMUL_RESOLVER
+	size_t nr_simd = crc32_x86_simd_update(X86_CRC32B, &crc, (const unsigned char *)p, nr);
+	nr -= nr_simd;
+	p += nr_simd;
+#endif
+
+	/* The trailing part */
+	for (; nr--; ++p) {
+		crc = ((crc >> 8) & 0x00FFFFFF) ^ crc32tab[(crc ^ (*p)) & 0xFF ];
+	}
+
+	return crc;
+}
+
+PHPAPI int php_crc32_stream_bulk_update(uint32_t *crc, php_stream *fp, size_t nr)
+{
+	size_t handled = 0, n;
+	char buf[1024];
+
+	while (handled < nr) {
+		n = nr - handled;
+		n = (n < sizeof(buf)) ? n : sizeof(buf); /* tweak to buf size */
+
+		n = php_stream_read(fp, buf, n);
+		if (n > 0) {
+			*crc = php_crc32_bulk_update(*crc, buf, n);
+			handled += n;
+		} else { /* EOF */
+			return FAILURE;
+		}
+	}
+
+	return SUCCESS;
+}
+
+/* {{{ Calculate the crc32 polynomial of a string */
+PHP_FUNCTION(crc32)
 {
 	char *p;
 	size_t nr;
-	uint32_t crcinit = 0;
-	register uint32_t crc;
+	uint32_t crc = php_crc32_bulk_init();
 
 	ZEND_PARSE_PARAMETERS_START(1, 1)
 		Z_PARAM_STRING(p, nr)
 	ZEND_PARSE_PARAMETERS_END();
 
-	crc = crcinit^0xFFFFFFFF;
+	crc = php_crc32_bulk_update(crc, p, nr);
 
-#if defined(__aarch64__) && defined(HAVE_SYS_AUXV_H)
-	if (has_crc32_insn()) {
-		crc = crc32_aarch64(crc, p, nr);
-		RETURN_LONG(crc^0xFFFFFFFF);
-	}
-#endif
-
-	for (; nr--; ++p) {
-		crc = ((crc >> 8) & 0x00FFFFFF) ^ crc32tab[(crc ^ (*p)) & 0xFF ];
-	}
-	RETURN_LONG(crc^0xFFFFFFFF);
+	RETURN_LONG(php_crc32_bulk_end(crc));
 }
 /* }}} */
